@@ -359,6 +359,12 @@ class KeyringCredentialStore(CredentialStore):
 
     B64MARKER = b"<B64>"
 
+    def __init__(self, credential_save_failed=None, fallback=False):
+        super(KeyringCredentialStore, self).__init__(credential_save_failed)
+        self._fallback = None
+        if fallback:
+            self._fallback = MemoryCredentialStore(credential_save_failed)
+
     @staticmethod
     def _ensure_keyring_imported():
         """Ensure the keyring module is imported (postponing side effects).
@@ -371,6 +377,12 @@ class KeyringCredentialStore(CredentialStore):
         if 'keyring' not in globals():
             global keyring
             import keyring
+        if 'NoKeyringError' not in globals():
+            global NoKeyringError
+            try:
+                from keyring.errors import NoKeyringError
+            except ImportError:
+                NoKeyringError = RuntimeError
 
     def do_save(self, credentials, unique_key):
         """Store newly-authorized credentials in the keyring."""
@@ -380,14 +392,36 @@ class KeyringCredentialStore(CredentialStore):
         # Gnome and KDE, when newlines are included in the password.  Avoid
         # this problem by base 64 encoding the serialized value.
         serialized = self.B64MARKER + b64encode(serialized)
-        keyring.set_password(
-            'launchpadlib', unique_key, serialized.decode('utf-8'))
+        try:
+            keyring.set_password(
+                'launchpadlib', unique_key, serialized.decode('utf-8'))
+        except NoKeyringError as e:
+            # keyring < 21.2.0 raises RuntimeError rather than anything more
+            # specific.  Make sure it's the exception we're interested in.
+            if (NoKeyringError == RuntimeError and
+                    'No recommended backend was available' not in str(e)):
+                raise
+            if self._fallback:
+                self._fallback.save(credentials, unique_key)
+            else:
+                raise
 
     def do_load(self, unique_key):
         """Retrieve credentials from the keyring."""
         self._ensure_keyring_imported()
-        credential_string = keyring.get_password(
-            'launchpadlib', unique_key)
+        try:
+            credential_string = keyring.get_password(
+                'launchpadlib', unique_key)
+        except NoKeyringError as e:
+            # keyring < 21.2.0 raises RuntimeError rather than anything more
+            # specific.  Make sure it's the exception we're interested in.
+            if (NoKeyringError == RuntimeError and
+                    'No recommended backend was available' not in str(e)):
+                raise
+            if self._fallback:
+                return self._fallback.load(unique_key)
+            else:
+                raise
         if credential_string is not None:
             if isinstance(credential_string, unicode_type):
                 credential_string = credential_string.encode('utf8')
@@ -434,6 +468,25 @@ class UnencryptedFileCredentialStore(CredentialStore):
         return None
 
 
+class MemoryCredentialStore(CredentialStore):
+    """CredentialStore that stores keys only in memory.
+
+    This can be used to provide a CredentialStore instance without
+    actually saving any key to persistent storage.
+    """
+    def __init__(self, credential_save_failed=None):
+        super(MemoryCredentialStore, self).__init__(credential_save_failed)
+        self._credentials = {}
+
+    def do_save(self, credentials, unique_key):
+        """Store the credentials in our dict"""
+        self._credentials[unique_key] = credentials
+
+    def do_load(self, unique_key):
+        """Retrieve the credentials from our dict"""
+        return self._credentials.get(unique_key)
+
+
 class RequestTokenAuthorizationEngine(object):
     """The superclass of all request token authorizers.
 
@@ -579,12 +632,75 @@ class RequestTokenAuthorizationEngine(object):
         raise NotImplementedError()
 
 
-class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
-    """The simplest (and, right now, the only) request token authorizer.
+class AuthorizeRequestTokenWithURL(RequestTokenAuthorizationEngine):
+    """Authorize using a URL.
+
+    This authorizer simply shows the URL for the user to open for
+    authorization, and waits until the server responds.
+    """
+
+    WAITING_FOR_USER = (
+        "Please open this authorization page:\n"
+        " (%s)\n"
+        "in your browser. Use your browser to authorize\n"
+        "this program to access Launchpad on your behalf.")
+    WAITING_FOR_LAUNCHPAD = (
+        "Press Enter after authorizing in your browser.")
+
+    def output(self, message):
+        """Display a message.
+
+        By default, prints the message to standard output. The message
+        does not require any user interaction--it's solely
+        informative.
+        """
+        print(message)
+
+    def notify_end_user_authorization_url(self, authorization_url):
+        """Notify the end-user of the URL."""
+        self.output(self.WAITING_FOR_USER % authorization_url)
+
+    def check_end_user_authorization(self, credentials):
+        """Check if the end-user authorized"""
+        try:
+            credentials.exchange_request_token_for_access_token(
+                self.web_root)
+        except HTTPError as e:
+            if e.response.status == 403:
+                # The user decided not to authorize this
+                # application.
+                raise EndUserDeclinedAuthorization(e.content)
+            else:
+                if e.response.status != 401:
+                    # There was an error accessing the server.
+                    print("Unexpected response from Launchpad:")
+                    print(e)
+                # The user has not made a decision yet.
+                raise EndUserNoAuthorization(e.content)
+        return credentials.access_token is not None
+
+    def wait_for_end_user_authorization(self, credentials):
+        """Wait for the end-user to authorize"""
+        self.output(self.WAITING_FOR_LAUNCHPAD)
+        stdin.readline()
+        self.check_end_user_authorization(credentials)
+
+    def make_end_user_authorize_token(self, credentials, request_token):
+        """Have the end-user authorize the token using a URL."""
+        authorization_url = self.authorization_url(request_token)
+        self.notify_end_user_authorization_url(authorization_url)
+        self.wait_for_end_user_authorization(credentials)
+
+
+class AuthorizeRequestTokenWithBrowser(AuthorizeRequestTokenWithURL):
+    """Authorize using a URL that pops-up automatically in a browser.
 
     This authorizer simply opens up the end-user's web browser to a
     Launchpad URL and lets the end-user authorize the request token
     themselves.
+
+    This is the same as its superclass, except this class also
+    performs the browser automatic opening of the URL.
     """
 
     WAITING_FOR_USER = (
@@ -594,10 +710,10 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
         "this program to access Launchpad on your behalf.")
     TIMEOUT_MESSAGE = "Press Enter to continue or wait (%d) seconds..."
     TIMEOUT = 5
-    WAITING_FOR_LAUNCHPAD = (
-            "Waiting to hear from Launchpad about your decision...")
     TERMINAL_BROWSERS = ('www-browser', 'links', 'links2', 'lynx',
                          'elinks', 'elinks-lite', 'netrik', 'w3m')
+    WAITING_FOR_LAUNCHPAD = (
+        "Waiting to hear from Launchpad about your decision...")
 
     def __init__(self, service_root, application_name, consumer_name=None,
                  credential_save_failed=None, allow_access_levels=None):
@@ -620,20 +736,10 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
               service_root, application_name, None,
               credential_save_failed)
 
-    def output(self, message):
-        """Display a message.
-
-        By default, prints the message to standard output. The message
-        does not require any user interaction--it's solely
-        informative.
-        """
-        print(message)
-
-    def make_end_user_authorize_token(self, credentials, request_token):
-        """Have the end-user authorize the token in their browser."""
-
-        authorization_url = self.authorization_url(request_token)
-        self.output(self.WAITING_FOR_USER % authorization_url)
+    def notify_end_user_authorization_url(self, authorization_url):
+        """Notify the end-user of the URL."""
+        super(AuthorizeRequestTokenWithBrowser,
+              self).notify_end_user_authorization_url(authorization_url)
 
         try:
             browser_obj = webbrowser.get()
@@ -651,28 +757,20 @@ class AuthorizeRequestTokenWithBrowser(RequestTokenAuthorizationEngine):
             if rlist:
                 stdin.readline()
 
-        self.output(self.WAITING_FOR_LAUNCHPAD)
         if browser_obj is not None:
             webbrowser.open(authorization_url)
+
+    def wait_for_end_user_authorization(self, credentials):
+        """Wait for the end-user to authorize"""
+        self.output(self.WAITING_FOR_LAUNCHPAD)
         start_time = time.time()
         while credentials.access_token is None:
             time.sleep(access_token_poll_time)
             try:
-                credentials.exchange_request_token_for_access_token(
-                    self.web_root)
-                break
-            except HTTPError as e:
-                if e.response.status == 403:
-                    # The user decided not to authorize this
-                    # application.
-                    raise EndUserDeclinedAuthorization(e.content)
-                elif e.response.status == 401:
-                    # The user has not made a decision yet.
-                    pass
-                else:
-                    # There was an error accessing the server.
-                    print("Unexpected response from Launchpad:")
-                    print(e)
+                if self.check_end_user_authorization(credentials):
+                    break
+            except EndUserNoAuthorization:
+                pass
             if time.time() >= start_time + access_token_poll_timeout:
                 raise TokenAuthorizationTimedOut(
                     "Timed out after %d seconds." % access_token_poll_timeout)
@@ -686,11 +784,23 @@ class RequestTokenAlreadyAuthorized(TokenAuthorizationException):
     pass
 
 
-class EndUserDeclinedAuthorization(TokenAuthorizationException):
+class EndUserAuthorizationFailed(TokenAuthorizationException):
+    """Superclass exception for all failures of end-user authorization"""
     pass
 
 
-class TokenAuthorizationTimedOut(TokenAuthorizationException):
+class EndUserDeclinedAuthorization(EndUserAuthorizationFailed):
+    """End-user declined authorization"""
+    pass
+
+
+class EndUserNoAuthorization(EndUserAuthorizationFailed):
+    """End-user did not perform any authorization"""
+    pass
+
+
+class TokenAuthorizationTimedOut(EndUserNoAuthorization):
+    """End-user did not perform any authorization in timeout period"""
     pass
 
 
