@@ -37,16 +37,16 @@ from lazr.uri import URI
 LPNET_SERVICE_ROOT = 'https://api.launchpad.net/'
 QASTAGING_SERVICE_ROOT = 'https://api.qastaging.launchpad.net/'
 STAGING_SERVICE_ROOT = 'https://api.staging.launchpad.net/'
-DEV_SERVICE_ROOT = 'https://api.launchpad.dev/'
+DEV_SERVICE_ROOT = 'https://api.launchpad.test/'
 DOGFOOD_SERVICE_ROOT = 'https://api.dogfood.paddev.net/'
-TEST_DEV_SERVICE_ROOT = 'http://api.launchpad.dev:8085/'
+TEST_DEV_SERVICE_ROOT = 'http://api.launchpad.test:8085/'
 
 LPNET_WEB_ROOT = 'https://launchpad.net/'
 QASTAGING_WEB_ROOT = 'https://qastaging.launchpad.net/'
 STAGING_WEB_ROOT = 'https://staging.launchpad.net/'
-DEV_WEB_ROOT = 'https://launchpad.dev/'
+DEV_WEB_ROOT = 'https://launchpad.test/'
 DOGFOOD_WEB_ROOT = 'https://dogfood.paddev.net/'
-TEST_DEV_WEB_ROOT = 'http://launchpad.dev:8085/'
+TEST_DEV_WEB_ROOT = 'http://launchpad.test:8085/'
 
 # If you use EDGE_SERVICE_ROOT, or its alias, or the equivalent
 # string, launchpadlib will issue a deprecation warning and use
